#ifndef _VisibilityOptionsManager_H__
#define _VisibilityOptionsManager_H__

#include "VisibilityManager.h"
#include "HierarchyInterface.h"
#include <OgreString.h>
#include <OgreStringVector.h>

namespace Ogre {
	
/**
	Helper class for setting and getting different visibility options.
*/
class VisibilityOptionsManager
{
public:
	/** Contructor.
		@param visManager current visibility manager handling
		the culling options
		@param hierarchInterface current hierarchy interface for 
		handling the traversal options
	*/
	VisibilityOptionsManager(GtpVisibility::VisibilityManager *visManager, 
							 GtpVisibility::HierarchyInterface *hierarchyInterface);
	
	/** Sets the given option for the scene traverser.
	    @remarks
		Options are:
		"Algorithm", "Threshold";			
	*/
	bool setOption( const String &, const void * );
	/** Gets the given option for the scene traverser.
	    @remarks
	    See setOption
	*/
	bool getOption( const String &, void * );
	bool getOptionKeys( StringVector &refKeys );
	
protected:
	GtpVisibility::VisibilityManager *mVisibilityManager;
	GtpVisibility::HierarchyInterface *mHierarchyInterface;
};

}
#endif // VisibilityOptionsManager_H